/* $Id: heartbeat.c,v 1.4 2008/03/21 11:22:08 simimeie Exp $
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <event.h>
#include "getstream.h"
#include "log.h"
#include "heartbeat.h"

char *				heartbeatpass = NULL;
char *				heartbeatserverstring = NULL;
static int			heartbeatserverport = 0;
static gs_sockaddr_struct	heartbeatserveraddr;
static struct event		hbevent;
static int			hbsock;

#define HBPACKETSIZE 1400
char hbpacket[HBPACKETSIZE];

int parse_heartbeat_parameter(char * optarg) {
	char * sepdot;
	
	if ((heartbeatpass != NULL) || (heartbeatserverstring != NULL)) {
		fprintf(stderr, "At most one heartbeat server can be defined.\n");
		return 0;
	}
	if ((sepdot = strchr(optarg, ':')) == NULL) {
		return 0;
	}
	if ((sepdot - optarg) > 16) {
		fprintf(stderr, "Due to packet size restrictions, the heartbeat\n");
		fprintf(stderr, " password must not be longer than 16 chars.\n");
		return 0;
	}
	heartbeatpass = calloc(1, (sepdot - optarg)+1);
	memcpy(heartbeatpass, optarg, (sepdot - optarg));
	heartbeatserverstring = strdup(sepdot+1);
	if ((sepdot = strrchr(heartbeatserverstring, ':')) == NULL) {
		heartbeatserverport = 4711;
	} else {
		heartbeatserverport = strtol(sepdot+1, NULL, 10);
		*sepdot = '\0'; /* Temporarily terminate the string here */
	}
	
#ifdef SUPPORTIPV6
	heartbeatserveraddr.sin6_family = GS_AF;
	if (!inet_pton(AF_INET6, heartbeatserverstring, &heartbeatserveraddr.sin6_addr)) {
		/* No V6 - try v4 */
		char v4addrinv6style[sizeof(heartbeatserverstring)+10];
		sprintf(v4addrinv6style, "::ffff:%s", heartbeatserverstring);
		if (!inet_pton(AF_INET6, v4addrinv6style, &heartbeatserveraddr.sin6_addr)) {
			fprintf(stderr, "ERROR: IP address '%s' is not valid.\n",
			                heartbeatserverstring);
			exit(-1);
		}
	}
#else /* ! IPV6 */
	heartbeatserveraddr.sin_family = GS_AF;
	if (!inet_aton(heartbeatserverstring, &heartbeatserveraddr.sin_addr)) {
		fprintf(stderr, "ERROR: IP address '%s' is not valid.\n",
		                heartbeatserverstring);
		exit(-1);
	}
#endif
	heartbeatserveraddr.GS_SINP = htons(heartbeatserverport);
	
	if (sepdot != NULL) {
		*sepdot = ':'; /* Restore the string */
	}
	return 1;
}

void doheartbeat(void) {
	int lensent;
	int lentosend = 0;
	struct stream * s = streams;
	struct transponder * t = NULL;
	unsigned long totclients = 0;
	unsigned long nstreams = 0;
	unsigned long nucstreams = 0;
	
	if (heartbeatserverstring == NULL) {
		return;
	}
	putlog(5, "Sending heartbeat.");
	while (s) {
		if (s->nuccs >= 0) {
			totclients += s->nuccs;
			nucstreams++;
		}
		nstreams++;
		s = s->next;
	}
#define printtohb(x...) \
	if (lentosend < (HBPACKETSIZE - 100)) { \
		lentosend += sprintf(&hbpacket[lentosend], x); \
	}
	printtohb("version = 0.1\n");
	printtohb("password = %s\n", heartbeatpass);
	printtohb("totclients = %lu\n", totclients);
	printtohb("pid = %u\n", getpid());
	printtohb("nucstreams = %lu\n", nucstreams);
	printtohb("nstreams = %lu\n", nstreams);
	if ((streams != NULL)
	 && ((t = streams->channel->transponder) != NULL)) {
		switch (t->type) {
		case DVB_S:	printtohb("transp = S %lu %c\n", t->tp.s.freq, t->tp.s.pol);
				break;
		case DVB_T:	printtohb("transp = T %lu", t->tp.t.freq);
				break;
		default:	break;
		};
	}
	
	lensent = send(hbsock, hbpacket, lentosend, MSG_DONTWAIT);
	if (lensent < lentosend) {
		putlog(4, "Heartbeat failed, sent only %d of %d bytes",
		          lensent, lentosend);
	}
}

static void hbtimerinit(void);
static void hbtimerexec(int fd, short event, void *arg) {
	doheartbeat();
	/* Reinit timer */
	hbtimerinit();
}

static void hbtimerinit(void) {
	struct timeval tv;
	
	tv.tv_usec = 0;
	tv.tv_sec = HEARTBEAT_INTERVAL;
	evtimer_set(&hbevent, hbtimerexec, &hbevent);
	evtimer_add(&hbevent, &tv);
}

int init_heartbeat(void) {
	if (heartbeatserverstring == NULL) {
		return 0;
	}
	
	hbsock = socket(GS_PF, SOCK_DGRAM, IPPROTO_UDP);
	if (hbsock < 0) {
		return 0;
	}
	setfdnonblocking(hbsock);
	if (connect(hbsock,
	            (struct sockaddr *)&heartbeatserveraddr,
	            sizeof(heartbeatserveraddr)) != 0) {
		return 0;
	}
	
	hbtimerinit();
	return 1;
}
